package com.okooo.osgi.demo;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;

import com.okooo.osgi.rpc.service.IRpcExporterService;

public class Activator implements BundleActivator {
	private HelloService helloService;
	private IRpcExporterService rpcExportService;
	protected Log logger = LogFactory.getLog(Activator.class);

	public void start(final BundleContext context) throws Exception {
		helloService = new HelloServiceImpl();
		context.registerService(HelloService.class.getName(), helloService,
				null);
		this.doExport(context);
	}

	public void stop(BundleContext context) throws Exception {
		this.unexport();
		helloService = null;
	}

	private void doExport(final BundleContext context)
			throws InvalidSyntaxException {
		this.export(context);
		context.addServiceListener(new ServiceListener() {
			@Override
			public void serviceChanged(ServiceEvent event) {
				if (event.getType() == ServiceEvent.REGISTERED) {
					unexport();
					export(context);
				} else if (event.getType() == ServiceEvent.UNREGISTERING) {
					unexport();
				}
			}
		}, "(objectClass=" + IRpcExporterService.class.getName() + ")");
	}

	private void export(final BundleContext context) {
		ServiceReference ref = context
				.getServiceReference(IRpcExporterService.class.getName());
		if (ref != null) {
			rpcExportService = (IRpcExporterService) context.getService(ref);
			// export HelloService as XML RPC service
			rpcExportService.export("HelloService", HelloService.class);
			System.out.println("export HelloService OK");
		} else {
			System.out.println("export HelloService ERROR");
		}
		ref = null;
	}

	private void unexport() {
		if (rpcExportService != null) {
			rpcExportService.unexport("HelloService");
			System.out.println("unexport HelloService OK");
		}
		rpcExportService = null;
	}

}
