package com.okooo.osgi.rpc;

import javax.servlet.Servlet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.service.http.HttpService;

import com.okooo.osgi.rpc.service.IRpcExporterService;
import com.okooo.osgi.rpc.service.impl.RpcExporterServiceImpl;

/**
 * 
 * @author jiangyubao@gmail.com
 * 
 */
public class Activator implements BundleActivator {
	private String url = "/xmlrpc";
	private String URL = "com.okooo.osgi.rpc.URL";

	protected Log logger = LogFactory.getLog(Activator.class);
	private BundleContext context;

	private ServiceReference ref;

	private Servlet servlet;
	private static Activator instance;
	private MyPropertyHandlerMapping propertyHandlerMapping;
	private IRpcExporterService rpcExportService;

	public static Activator getInstance() {
		return instance;
	}

	public Object getService(Class clazz) {
		ServiceReference sref = context.getServiceReference(clazz.getName());
		if (sref != null) {
			return context.getService(sref);
		}
		return null;
	}

	private void registerServlet() {
		if (ref == null) {
			ref = context.getServiceReference(HttpService.class.getName());
		}
		if (ref != null) {
			try {
				HttpService http = (HttpService) context.getService(ref);
				http.registerServlet(url, servlet, null, null);
			} catch (Exception e) {
				e.printStackTrace();
				logger.error("", e);
			}
		}
	}

	private void unregisterServlet() {
		if (ref != null) {
			try {
				HttpService http = (HttpService) context.getService(ref);
				http.unregister("/xmlrpc");
			} catch (Exception e) {
				e.printStackTrace();
				logger.error("", e);
			}
		}
	}

	@Override
	public void start(BundleContext ctx) throws Exception {
		instance = this;
		String urlParam = ctx.getProperty(URL);
		if (urlParam != null) {
			url = urlParam;
		}
		propertyHandlerMapping = new MyPropertyHandlerMapping();
		context = ctx;
		servlet = new MyXmlRpcServlet();
		registerServlet();
		context.addServiceListener(new ServiceListener() {
			@Override
			public void serviceChanged(ServiceEvent event) {
				switch (event.getType()) {
				case ServiceEvent.REGISTERED:
					registerServlet();
					break;
				case ServiceEvent.UNREGISTERING:
					unregisterServlet();
					break;
				}
			}
		}, "(objectClass=" + HttpService.class.getName() + ")");
		rpcExportService = new RpcExporterServiceImpl();
		ctx.registerService(IRpcExporterService.class.getName(),
				rpcExportService, null);
	}

	@Override
	public void stop(BundleContext ctx) throws Exception {
		unregisterServlet();
		ref = null;
		servlet = null;
		rpcExportService = null;
	}

	public MyPropertyHandlerMapping getPropertyHandlerMapping() {
		return propertyHandlerMapping;
	}

	public BundleContext getContext() {
		return context;
	}

}
